import React, { useState, useEffect } from 'react';
import { Node, dataConfig, AugmentedNodeAttributes } from './types';
import { Button } from '../../../../components/buttons';

function getUniqueValues(arr: any[]): any[] {
  return [...new Set(arr)];
}

interface ConfigPanelProps {
  data: AugmentedNodeAttributes[];
  onUpdateData: (data: dataConfig) => void;
}

const ConfigPanel: React.FC<ConfigPanelProps> = ({ data, onUpdateData }) => {
  const [state, setState] = useState<{
    entityVertical: string;
    attributeEntity: string;
    attributeValueSelected: string;
    orderNameXaxis: string;
    orderNameYaxis: string;
    isButtonEnabled: boolean;
  }>({
    entityVertical: '',
    attributeEntity: '',
    attributeValueSelected: '',
    orderNameXaxis: '',
    orderNameYaxis: '',
    isButtonEnabled: true,
  });

  const nodeLabels: string[] = data.map((node: any) => node.label);

  const uniqueNodeLabels = getUniqueValues(nodeLabels);

  const entityOptions = [...uniqueNodeLabels].map((value, index) => (
    <option key={`option${index}`} value={value}>
      {value}
    </option>
  ));

  // Extract unique attributeEntity values from the entire data array
  const [attributeEntityMenuItems, setattributeEntityMenuItems] = useState<string[]>([]);
  // Filter the data based on the selected entity (label)
  const [filteredData, setFilteredData] = useState<AugmentedNodeAttributes[]>([]);
  const [attributeOptions, setAttributeOptions] = useState<any[]>([]);

  useEffect(() => {
    if (state.entityVertical) {
      const selectedEntity = state.entityVertical;
      // Filter the data based on the selected entity (label)
      const filteredData: AugmentedNodeAttributes[] = data.filter((item) => item.label === selectedEntity);

      setFilteredData(filteredData);
    } else {
      setFilteredData([]);
    }
  }, [state.entityVertical, data]);

  useEffect(() => {
    if (filteredData.length > 0) {
      const attributes: object = filteredData[0].attributes;

      if (attributes) {
        const keys = Object.keys(attributes);
        setattributeEntityMenuItems(keys);
      }
    } else {
      setattributeEntityMenuItems([]); // Clear the attributeEntityMenuItems when there's no filtered data
    }
  }, [filteredData]);

  useEffect(() => {
    // Update attributeOptions when attributeEntity changes
    if (filteredData.length > 0) {
      const filteredAAttributes: any[] = (filteredData as AugmentedNodeAttributes[]).map((item: AugmentedNodeAttributes) => {
        const attributeValueSelected = item.attributes[state.attributeEntity];

        if (Array.isArray(attributeValueSelected)) {
          if (attributeValueSelected.length === 1) {
            return attributeValueSelected[0];
          } else {
            return attributeValueSelected.join('-');
          }
        } else if (typeof attributeValueSelected === 'string' || typeof attributeValueSelected === 'number') {
          return attributeValueSelected;
        } else {
          return null; // Return null for other types
        }
      });

      if (filteredAAttributes) {
        // Extract unique values from the relation's attributes
        const uniqueValues = Array.from(new Set(filteredAAttributes));
        const firstElement = uniqueValues[0];

        if (typeof firstElement === 'number') {
          // Sort numbers in descending order
          const sortedValues = uniqueValues.slice().sort((a, b) => a - b);

          setAttributeOptions(sortedValues);
        } else if (typeof firstElement === 'string') {
          // Sort strings in descending order
          // localCompare is useful to take into account local language consideration.
          // but breaks for comparing an URL
          const sortedValues = uniqueValues.slice().sort((a, b) => a - b);
          setAttributeOptions(sortedValues);
        } else {
          // Handle other data types as needed
          setAttributeOptions(uniqueValues); // Clear attributeOptions for unsupported data types
        }
      }
    } else {
      setAttributeOptions([]); // Clear attributeOptions when there's no filtered data
    }
  }, [state.attributeEntity, filteredData]);

  const onClickMakeButton = () => {
    // Retrieve the selected values

    const { entityVertical, attributeEntity, attributeValueSelected, orderNameXaxis, orderNameYaxis, isButtonEnabled } = state;
    const isAxisSelected = orderNameXaxis !== '' || orderNameYaxis !== '';

    // Call the callback to send the data to the parent component (VisSemanticSubstrates)
    if (isAxisSelected) {
      onUpdateData({ entityVertical, attributeEntity, attributeValueSelected, orderNameXaxis, orderNameYaxis, isButtonEnabled });
    }
  };

  return (
    <div className="nav card w-full">
      <div className="card-body flex flex-row overflow-y-auto max-w-[60vw] self-center items-center">
        <div className="select-container">
          <label className="select-label">Entity:</label>
          <select
            className="select"
            id="standard-select-entity"
            value={state.entityVertical}
            onChange={(e) => setState({ ...state, entityVertical: e.target.value })}
          >
            <option value="" disabled>
              Select an entity
            </option>
            {entityOptions}
          </select>
        </div>

        <div className="select-container">
          <label className="select-label">Attribute:</label>
          <select
            className={`select ${attributeEntityMenuItems.length === 0 ? 'select-disabled' : ''}`}
            id="standard-select-relation"
            value={state.attributeEntity}
            onChange={(e) => setState({ ...state, attributeEntity: e.target.value, attributeValueSelected: '', orderNameXaxis: '' })}
          >
            <option value="" disabled>
              Select a relation
            </option>
            {attributeEntityMenuItems.map((value, index) => (
              <option key={`option${index}`} value={value}>
                {value}
              </option>
            ))}
          </select>
        </div>

        <div className="select-container">
          <label className="select-label">Selected Attribute:</label>
          <select
            className={`select ${attributeOptions.length === 0 ? 'select-disabled' : ''}`}
            id="standard-select-attribute"
            value={state.attributeValueSelected}
            onChange={(e) => setState({ ...state, attributeValueSelected: e.target.value, orderNameXaxis: '', orderNameYaxis: '' })}
          >
            <option value="" disabled>
              Select an attribute
            </option>
            {attributeOptions.map((value, index) => (
              <option key={`option${index}`} value={value}>
                {value}
              </option>
            ))}
          </select>
        </div>

        <div className="select-container">
          <label className="select-label">X-axis:</label>
          <select
            className={`select ${attributeEntityMenuItems.length === 0 ? 'select-disabled' : ''}`}
            id="standard-select-relation"
            value={state.orderNameXaxis}
            onChange={(e) => setState({ ...state, orderNameXaxis: e.target.value })}
          >
            <option value="" disabled>
              Select a x axis
            </option>
            {attributeEntityMenuItems.map((value, index) => (
              <option key={`option${index}`} value={value}>
                {value}
              </option>
            ))}
          </select>
        </div>

        <div className="select-container">
          <label className="select-label">Y-axis:</label>
          <select
            className={`select ${attributeEntityMenuItems.length === 0 ? 'select-disabled' : ''}`}
            id="standard-select-relation"
            value={state.orderNameYaxis}
            onChange={(e) => setState({ ...state, orderNameYaxis: e.target.value })}
          >
            <option value="" disabled>
              Select a y axis
            </option>
            {attributeEntityMenuItems.map((value, index) => (
              <option key={`option${index}`} value={value}>
                {value}
              </option>
            ))}
          </select>
        </div>

        <Button
          label="Make"
          type="secondary"
          variant="solid"
          disabled={!state.isButtonEnabled || (!state.orderNameXaxis && !state.orderNameYaxis)}
          onClick={onClickMakeButton}
        />
      </div>
    </div>
  );
};

export default ConfigPanel;
